// copied from bamps/src/projects/grhd/initialdata/michel.c

#include <math.h>
#include <stdlib.h>
#include <stdio.h>

void main(int argc, char *argv[]) {

    double M = 1.0;
    double rc = 200.0*M;
    double rhoc = 7e-4;
    double gamma = 4.0/3.0;

    double urc   = sqrt(.5 * M / rc);
    double Vc2   = urc*urc / (1.-3.*urc*urc);

    double K     = Vc2*(gamma-1.)*pow(rhoc,1.-gamma) / (gamma*(-Vc2+gamma-1.));

    double hc    = 1. + gamma / (gamma-1.) * K * pow(rhoc,gamma-1.);
    double c1    = rhoc * urc * rc * rc;
    double c3    = hc * hc * (urc*urc + 1. - 2.*M / rc);

    int nnn = 1000;
    int i;
    double r;
    double rmin = 1.8*M, rmax = 200.0*M;
    double dr = (rmax-rmin)/(nnn-1);

    double *gr      = malloc(nnn*sizeof(double));
    double *grho    = malloc(nnn*sizeof(double));
    double *geps    = malloc(nnn*sizeof(double));
    double *gp      = malloc(nnn*sizeof(double));
    double *gvr     = malloc(nnn*sizeof(double));
    double *gur     = malloc(nnn*sizeof(double));
    double *gut2    = malloc(nnn*sizeof(double));

    for(i=0; i<nnn; i++) {

      r = rmax - dr*i;
      gr[i] = r;

      double ur    = sqrt(fabs(-1. + 2.*M / r) + 0.01);
      double ut2, h, f, df;

      int it=0;
      for(it=0;it<100;it++) {

        ut2 = (ur*ur+1.-2.*M/r);
        h   = 1. + (gamma / (gamma-1.)) * K * pow(c1 / (ur*r*r), gamma-1.);
        f   = h*h*ut2 - c3;
        df = 2. * h * (-(h-1.)*(gamma-1.)/ur) * ut2 + 2. * h * h * ur;

        ur = ur - f/df;

        if(fabs(f/df)<1.e-14) break;

      }

      grho[i] = c1 / (ur*r*r);
      gp[i]   = K * pow(grho[i],gamma);
      geps[i] = gp[i] / (grho[i]*(gamma-1.));

      ut2 = (ur*ur+1.-2.*M/r);
      gur[i] = ur;
      gut2[i] = ut2;

      double alpha = sqrt(r / (r + 2.*M));
      double betar = 2.*M / (r + 2.*M);
      double ut    = (-2.*M * ur / r + sqrt(ut2)) / (-1.+2.*M/r);
      double vr    = ur / (alpha*ut) + betar / alpha;

      gvr[i] = vr;

    }


    FILE *io = fopen("bondi-acc-ref.txt", "w");
    if (!io) {
      printf("FAILED\n");
      exit(-1);
    }
    for (i=nnn-1; i>=0; i--) {
      fprintf(io, "% .6e % .6e % .6e % .6e % .6e % .6e % .6e\n", gr[i], grho[i], geps[i], gp[i], gvr[i], gur[i], gut2[i]);
    }
    fclose(io); io = NULL;

    free(grho); grho = NULL;
    free(geps); geps = NULL;
    free(gp);   gp = NULL;
    free(gvr);  gvr = NULL;
    free(gr);   gr = NULL;
    free(gur);  gur = NULL;
    free(gut2); gut2 = NULL;

}
