using dg1d
using dg1d.Jacobi
using GLMakie
using LinearAlgebra


let
if true

# bs = dg1d.Bspline.Bspline2(5)
x, _ = dg1d.LGL.rule(5)
bs = dg1d.Bspline.Bspline1(x)
Npts = bs.Npts
xx = collect(range(-2.0,2.0,length=300))
Bs = [ dg1d.Bspline.polynomial(xx[i], j, bs) for i in 1:length(xx), j in 1:Npts ]
∂Bs = [ dg1d.Bspline.derivative_polynomial(xx[i], j, bs) for i in 1:length(xx), j in 1:Npts ]

Vs = [ dg1d.Bspline.polynomial(bs.xs[2+i], j, bs) for i in 1:bs.Npts, j in 1:Npts ]
# display(Vs)
∂Vs = [ dg1d.Bspline.derivative_polynomial(bs.xs[2+i], j, bs) for i in 1:bs.Npts, j in 1:Npts ]
# display(∂Vs)

M = dg1d.Bspline.mass_matrix(bs)
S = dg1d.Bspline.stiffness_matrix(bs)
ST = transpose(S)
BB = similar(M); fill!(BB, 0.0)
o = 2
for i in 1:Npts, j in 1:Npts
  for k in 1:bs.Npts-1
    xl, xr = bs.xs[o+k], bs.xs[o+k+1]
    Nil, Nir = dg1d.Bspline.polynomial(xl, i, bs), dg1d.Bspline.polynomial(xr, i, bs)
    Njl, Njr = dg1d.Bspline.polynomial(xl, j, bs), dg1d.Bspline.polynomial(xr, j, bs)
    BB[i,j] += Nir*Njr - Nil*Njl
  end
end
RBR = dg1d.purge_zeros(S+ST)
display(M)
display(S)
display(RBR)
display(BB)

if any(isnan, Bs)
  error("polynomial produces NaNs")
end
if any(isnan, ∂Bs)
  error("derivative_polynomial produces NaNs")
end
if any(isnan, Vs)
  error("polynomial produces NaNs")
end
if any(isnan, ∂Vs)
  error("derivative_polynomial produces NaNs")
end

fig = Figure()
ax = Axis(fig[1,1])
vlines!(ax, bs.xs, color=(:black,0.5), linestyle=:dash)

for (i,(B,V)) in enumerate(zip(eachcol(Bs),eachcol(Vs)))
  lines!(ax, xx, B, label="$i", color=Makie.wong_colors()[i])
  scatter!(ax, bs.xs[3:2+bs.Npts], V, color=Makie.wong_colors()[i])
end
PoU = vec(sum(Bs, dims=2))
lines!(ax, xx, PoU, label=L"\Sigma_n B_n", linestyle=:dot)

# for (i,(∂B,∂V)) in enumerate(zip(eachcol(∂Bs),eachcol(∂Vs)))
#   lines!(ax, xx, ∂B, label="$i", linestyle=:dash, color=Makie.wong_colors()[i])
#   scatter!(ax, bs.xs[3:2+bs.Npts], ∂V, color=Makie.wong_colors()[i])
# end

# FD_∂B = diff(Bs, dims=1) ./ diff(xx)
# xx_ctr = xx[1:end-1] .+ (xx[2]-xx[1])/2
# for (i,∂B) in enumerate(eachcol(FD_∂B))
#   lines!(ax, xx_ctr, ∂B, label="$i", color=Makie.wong_colors()[i])
# end

axislegend(ax)
display(fig)

end # if
end


let
if false
# # fn = x -> x < 0.15 ? -x+1 : 0.0
# # fn = x -> x < 0.0 ? -x : 0.0
Npts = 5
xs, _ = dg1d.LGL.rule(Npts)
xx = collect(range(extrema(xs)...,length=100))
fn = x -> x < -0.5 ? -x+1 : 0.0
fx = fn.(xs)

T = zeros(Float64, Npts+1, Npts)
T[1,1] = T[end,end] = 1.0
for i in 2:Npts
  T[i,i-1:i] .= 0.5
end
pinvT = pinv(T)
# display(T)
# display(pinvT)

knots = vcat(xs[1], (xs[1:end-1].+xs[2:end])./2, xs[end])
# avg_fx = vcat(fx[1], (fx[1:end-1].+fx[2:end])./2, fx[end])
avg_fx = T * fx
bs = dg1d.Bspline.Bspline2(xs)
Vs = dg1d.Bspline.vandermonde_matrix(knots, bs)
intrpmat_Bs = dg1d.Bspline.vandermonde_matrix(xx, bs)
A_lsqr = transpose(Vs)*Vs
b_lsqr = transpose(Vs)*avg_fx
intrpmat_Bs_to_collocs = dg1d.Bspline.vandermonde_matrix(xs, bs)
pinv_intrpmat_Bs_to_collocs = pinv(intrpmat_Bs_to_collocs)
# display(intrpmat_Bs_to_collocs)
# display(pinv_intrpmat_Bs_to_collocs)
# display(pinv_intrpmat_Bs_to_collocs*intrpmat_Bs_to_collocs)
# display(intrpmat_Bs_to_collocs*pinv_intrpmat_Bs_to_collocs)

# nodal Bspline
nodal_Bs = Vs \ avg_fx
fn_nodal_Bs = intrpmat_Bs * nodal_Bs

# quasi-nodal Bspline
quasi_nodal_Bs = avg_fx
fn_quasi_nodal_Bs = intrpmat_Bs * quasi_nodal_Bs
# quasi-nodal Bspline data
quasi_nodal_Bs_samples = Vs * quasi_nodal_Bs
# corrected interpolateD
c_quasi_nodal_Bs_samples = copy(quasi_nodal_Bs_samples)
# c_quasi_nodal_Bs_samples[3] = 0
c_quasi_nodal_Bs_samples[3] /= 10
c_quasi_nodal_Bs = Vs \ c_quasi_nodal_Bs_samples
c_quasi_nodal_Bs = max.(c_quasi_nodal_Bs, 0.0)
c_quasi_nodal_Bs_samples = Vs * c_quasi_nodal_Bs
fn_c_quasi_nodal_Bs = intrpmat_Bs * c_quasi_nodal_Bs

cc_quasi_nodal_Bs = copy(quasi_nodal_Bs)
cc_quasi_nodal_Bs[3] = c_quasi_nodal_Bs[3]
fn_cc_quasi_nodal_Bs = intrpmat_Bs * cc_quasi_nodal_Bs

# corrected nodal Bspline
c_nodal_Bs = max.(nodal_Bs, 0.0)
fn_c_nodal_Bs = intrpmat_Bs * c_nodal_Bs

# least-square Bspline
lsqr_Bs = A_lsqr \ b_lsqr
fn_lsqr_Bs = intrpmat_Bs * lsqr_Bs

# corrected least-sqaure Bspline
c_lsqr_Bs = max.(lsqr_Bs, 0.0)
fn_c_lsqr_Bs = intrpmat_Bs * c_lsqr_Bs

# nodal Lagrange
intrp_La = dg1d.lagrange_interp_matrix(xx, xs)
fn_La = intrp_La * fx

# modal Legendre
Ln = dg1d.Legendre.projection(fn, Npts)
fn_Ln = dg1d.Legendre.interpolate.(xx, Ref(Ln))

fig = Figure()
ax = Axis(fig[1,1])

vlines!(ax, xs, color=(:black,0.4), linestyle=:dash)
# lines!(ax, xx, fn_Ln, color=:blue, label="Legendre")
lines!(ax, xx, fn_La, color=:orange, label="Lagrange")
scatterlines!(ax, xs, fx, color=:orange, markersize=22, linestyle=:dash)

# lines!(ax, xx, fn_nodal_Bs, color=:blue, label="nodal Bspline")
# scatterlines!(ax, knots, nodal_Bs, color=:blue, markersize=18, linestyle=:dash)

lines!(ax, xx, fn_quasi_nodal_Bs, color=:green, label="quasi-nodal Bspline")
scatterlines!(ax, knots, quasi_nodal_Bs, color=:green, markersize=14, linestyle=:dash)

scatter!(ax, knots, quasi_nodal_Bs_samples, color=:brown, markersize=10,
              label="quasi-nodal Bspline samples")
scatter!(ax, knots, c_quasi_nodal_Bs_samples, color=:cyan, markersize=8,
              label="corrected quasi-nodal Bspline samples")
lines!(ax, xx, fn_c_quasi_nodal_Bs, color=:magenta, label="corrected quasi-nodal Bspline")
scatterlines!(ax, knots, c_quasi_nodal_Bs, color=:magenta, markersize=6, linestyle=:dash)

lines!(ax, xx, fn_cc_quasi_nodal_Bs, color=:limegreen, label="a posteriori corrected quasi-nodal Bspline")
scatterlines!(ax, knots, cc_quasi_nodal_Bs, color=:limegreen, markersize=6, linestyle=:dash)

lines!(ax, xx, fn_c_nodal_Bs, color=:red, label="corrected nodal Bspline")
scatterlines!(ax, knots, c_nodal_Bs, color=:red, markersize=6, linestyle=:dash)

# lines!(ax, xx, fn_lsqr_Bs, color=:violet, label="least-square Bspline")
# scatterlines!(ax, knots, lsqr_Bs, color=:violet, markersize=14, linestyle=:dash)

# lines!(ax, xx, fn_c_lsqr_Bs, color=:purple, label="corrected least-square Bspline")
# scatterlines!(ax, knots, c_lsqr_Bs, color=:purple, markersize=14, linestyle=:dash)

axislegend(ax)
display(fig)

end # if
end
