Project(env::Environment, prms) = Project(env, env.mesh, prms)


function Project(env::Environment, mesh::Mesh1d, prms)

  equation = make_Equation(prms["HeatEq"], dimension(mesh))

  bdryconds = dg1d.DirichletBC2()
  P = Project(equation, bdryconds)

  # register variables
  # TODO Somehow replace _register_variables! with register_variables!
  @unpack cache = env
  _register_variables!(cache, equation)
  display(cache)

  # setup initial data
  initialdata!(env, P, prms["HeatEq"])

  return P, (bdryconds, nothing, nothing)
end


#######################################################################
#                         Register variables                          #
#######################################################################


function _register_variables!(cache, eq::AbstractHeatEq)
  register_variables!(cache,
    dynamic_variablenames = (:u,),
    rhs_variablenames     = (:rhs_u,),
    static_variablenames  = (:flx_u, :q, :flx_q, :v),
    cell_variablenames    = (:max_v,),
    bdry_variablenames    = (:nflx_u,:nflx_q,:bdry_u,:bdry_q,:bdry_v),
    global_variablenames  = (:t,))
end


function _register_variables!(cache, bdryconds::Nothing) end
function _register_variables!(cache, bdryconds::dg1d.AbstractBC)
  register_variables!(cache,
    static_variablenames  = (:init_u, :init_flx_u, :init_v)
  )
end
