#######################################################################
#                                Type                                 #
#######################################################################


Base.@kwdef struct EntropyProduction{T_Mesh<:Mesh} <: AbstractTCI
  mesh::T_Mesh
  # TODO Rename to max_EP
  EP_max::Float64
end


#######################################################################
#                               Methods                               #
#######################################################################


function compute_indicator!(
    flag, EP, EPjump_lhs, EPjump_rhs, # mutated outputs
    E, Em1, F, Fm1, t, tm1,           # inputs
    tci::EntropyProduction)

  @unpack mesh       = tci
  @unpack invdetJ, x = get_static_variables(mesh.cache)
  @unpack D          = mesh.element
  L                  = layout(mesh)
  Npts, K            = L

  E       = dg1d.vreshape(E, L)
  F       = dg1d.vreshape(F, L)
  Em1     = dg1d.vreshape(Em1, L)
  Fm1     = dg1d.vreshape(Fm1, L)
  EP      = dg1d.vreshape(EP, L)
  x       = dg1d.vreshape(x, L)
  invdetJ = dg1d.vreshape(invdetJ, L)

  # entropy residual
  dt = t - tm1
  dt <= 0 && return # occurs on first time step when no previous entropy is available

  # use inplace operations
  @. Fm1 += F
  mul!(EP, D, Fm1, 0.5, false)
  @. EP *= invdetJ
  @. EP += (E - Em1) / dt
  @. EP = abs(EP)
  # EP .= abs.((E - Em1) / dt + 0.5 * D * (F + Fm1) .* invdetJ)

  # # entropy jumps
  # for k = 1:K
  #   # following index convention of numericalflux.jl
  #   k_lhs, k_rhs = periodic_index(k-1, K), periodic_index(k, K)
  #   F_lhs, F_rhs = F[end, k_lhs], F[1,k_rhs]
  #   EPjump = abs(F_lhs - F_rhs)
  #   EPjump_lhs[k_rhs], EPjump_rhs[k_lhs] = EPjump, EPjump
  # end

  # What parameters can be used?
  # See docs/src/hrsc-tci.md for a discussion
  # One possibility for a scale could be the grid average entropy.
  # avg_E = dg1d.grid_average(E, mesh) # use as scale for indicator
  # @assert !(isapprox(avg_E, 0.0)) "No scale for entropy production available!"

  ep_max = maximum(abs, EP)

  @unpack EP_max = tci
  for k = 1:K
    # cell_EP = maximum( (maximum(EP[:,k]), EPjump_lhs[k], EPjump_rhs[k]) )
    # cell_EP = minimum((cell_EP, EP_max))
    flag[k] = @views maximum(EP[:,k]) / ep_max
  end

  return
end


function compute_indicator!(
    flag, EP,
    tci::EntropyProduction)

  @unpack mesh       = tci
  @unpack invdetJ, x = get_static_variables(mesh.cache)
  @unpack D          = mesh.element
  L                  = layout(mesh)
  Npts, K            = L

  EP      = dg1d.vreshape(EP, L)

  # What parameters can be used?
  # See docs/src/hrsc-tci.md for a discussion
  # One possibility for a scale could be the grid average entropy.
  # avg_E = dg1d.grid_average(E, mesh) # use as scale for indicator
  # @assert !(isapprox(avg_E, 0.0)) "No scale for entropy production available!"

  ep_max = maximum(abs, EP)

  @unpack EP_max = tci
  for k = 1:K
    flag[k] = @views maximum(EP[:,k]) / ep_max
  end

  return
end
