module Sandbox


const BLACKLISTED_TOKENS = [ "function", "using", "import", "struct", "mutable",
                             "eval", "macro", "@", "begin", "end", "while", "for",
                             "if", "else", "elseif", "try", "catch", "Main", "module",
                             "dg1d", "println", "print", "display", "show" ]

const ROOTDIR = joinpath(@__DIR__, "..")


# rudimentary protection against arbitrary code execution
function has_blacklisted_tokens(str)
  lstr = lowercase(str)
  return any(occursin.(BLACKLISTED_TOKENS, Ref(str)))
end


function parse_expression_string(str::AbstractString)
  expr_str = strip(str)
  m = match(r"\$\((.*)\)$"s #= g for . to match \n =#, expr_str)
  if isnothing(m)
    return str
  else
    lines = split(strip(m.captures[1]), '\n')
    if any(has_blacklisted_tokens.(lines))
      error("Expression string contains blacklisted tokens!")
    end
    try
      exprs = Meta.parse.(lines)
      result = eval.(exprs)
      return result[end]
    catch
      error("Failed to parse expression string '$(str)'")
    end
  end
end
parse_expression_string(x) = x


end
