mutable struct VTKOutput
    dir::String
    meshcount::Int64 # number of meshes written
    tcount::Int64 # number of time steps written
end


function VTKOutput(dir)
    isdir(dir) && throw(ArgumentError("directory '$dir' already exists"))
    mkdir(dir)
    pvd = paraview_collection(joinpath(dir,"timeseries"))
    vtk_save(pvd)
    return VTKOutput(dir, 0, 0)
end


function vtkwrite_scalar(out::VTKOutput, mesh::Mesh, t, vars, names)

    tcount = out.tcount + 1
    celliters = [ eachcell(mesh, var) for var in vars ]

    @unpack x, y = get_static_variables(mesh.cache)
    z = zeros(Float64, length(x))

    @unpack element = mesh
    @unpack Npts = element
    Nx, Ny = Npts, Npts
    LI = LinearIndices((Nx,Ny))

    paraview_collection(joinpath(out.dir,"timeseries"), append=true) do pvd
        pts = vcat(x',y')
        cells = WriteVTK.VTKBase.AbstractMeshCell[]
        eNx, eNy = Nx-2, Ny-2
        for (k,o) in enumerate(mesh.offsets)
            idxs = zeros(Int64, Nx*Ny)
            # corners
            idxs[1] = LI[1,1]
            idxs[2] = LI[end,1]
            idxs[4] = LI[1,end]
            idxs[3] = LI[end,end]
            # edges
            i = 4
            if eNx > 0
                for ii = 1:eNx
                    idxs[i+ii] = LI[1+ii,1]
                end
                # idxs[i+1:i+eNx+1] .= LI[2:end-1,1]
                i += eNx
            end
            if eNy > 0
                for ii = 1:eNy
                    idxs[i+ii] = LI[end,1+ii]
                end
                # idxs[i+1:i+eNy+1] .= LI[end,2:end-1]
                i += eNy
            end
            if eNx > 0
                for ii = 1:eNx
                    idxs[i+ii] = LI[1+ii,end]
                end
                # idxs[i+1:i+eNx+1] .= LI[2:end-1,end]
                i += eNx
            end
            if eNy > 0
                for ii = 1:eNy
                    idxs[i+ii] = LI[1,1+ii]
                end
                # idxs[i+1:i+eNy+1] .= LI[1,2:end-1]
                i += eNy
            end
            # bulk
            if eNx > 0 && eNy > 0
                c = 1
                for jj = 1:eNy, ii = 1:eNx
                    idxs[i+c] = LI[1+ii,1+jj]
                    c += 1
                end
                # idxs[i+1:end] .= LI[2:end-1,2:end-1][:]
            end
            idxs .+= o
            push!(cells, MeshCell(VTKCellTypes.VTK_LAGRANGE_QUADRILATERAL, idxs))
        end
        vtk_grid(joinpath(out.dir, "data$tcount"), pts, cells) do vtk
            for (name,var) in zip(names,vars)
                vtk[name, VTKPointData()] = var
            end
            pvd[t] = vtk
        end
    end

    # updating count after all writes succeeded
    out.tcount = tcount

end
