@testset "Variables" begin


@testset "Variables" begin


@testset "Constructors" begin

  ### usage examples

  npts = 10
  # holds no variables, but any added ones must be of length npts
  vars = dg1d.Variables(npts)
  # holds variables a, b, abc_def, data initialized to zero
  # note that Symbols have a restricted syntax, hence, try to stick to
  # standard characters, digits and underscores
  vars = dg1d.Variables(npts, :a, :b, :abc_def)

  # we can also wrap variables around already given data
  npts = 10
  data = randn(5)
  vars = dg1d.Variables(data, npts, :a, :b)

  ### tests
  npts = -5
  @test_throws ArgumentError dg1d.Variables(npts, :a, :b) # npts > 0

end


@testset "Manipulations" begin

  npts = 10
  vars = dg1d.Variables(npts)

  # add elements add
  dg1d.add!(vars, :a, randn(10))
  dg1d.add!(vars, :b, randn(10))

  # register elements =^= add variable and initialize it to zero
  dg1d.register!(vars, :c)
  dg1d.register!(vars, [:d, :e])

  # get names with dg1d.names
  varnames = dg1d.names(vars)
  @test all(sort(varnames) .== [ :a, :b, :c, :d, :e ])

  # trying to add! a variable that is already present throws
  @test_throws ErrorException dg1d.add!(vars, :a, randn(10))

  # elements can be access with dotting and strings
  @test vars.a == vars.datadict[:a]

  # add or overwrite elements with set!
  # this previously failed with add!
  dg1d.set!(vars, :a, randn(10))

  # you can also replace all data by aliasing it to a single vector
  nvars = dg1d.n_variables(vars)
  newdata = randn(npts*nvars)
  dg1d.set!(vars, newdata)

  # remove values with remove!
  dg1d.remove!(vars, :a)
  varnames = dg1d.names(vars)
  @test all(sort(varnames) .== [ :b, :c, :d, :e ]) # only b, c and d should be left
  # removing a non-existent name will throw
  @test_throws ErrorException dg1d.remove!(vars, :a)

  # remove elements with unset! when you want to avoid such errors
  dg1d.unset!(vars, :b)
  dg1d.unset!(vars, :b)
  varnames = dg1d.names(vars)
  @test all(sort(varnames) .== [ :c, :d, :e ]) # only c and d should be left

  # redo above tests but with wrapped data
  data = randn(2*npts)
  vars = dg1d.Variables(data, npts, :a, :b)
  init_varnames = dg1d.names(vars)

end


@testset "Checks" begin

  npts = 10
  vars = dg1d.Variables(npts, :a, :b, :c)

  @test isinf(vars) == false
  @test isnan(vars) == false

  vars.a[1] = Inf
  vars.b[1] = NaN

  @test isinf(vars) == true
  @test isnan(vars) == true

  @test npts == length(vars)

  @test isregistered(vars, :a)
  @test isregistered(vars, :b)
  @test isregistered(vars, :c)
  @test !isregistered(vars, :d)

end


end # Variables


end # Variables
